@@ -1,5 +1,5 @@
 /*
-% Copyright (C) 2003-2016 GraphicsMagick Group
+% Copyright (C) 2003-2018 GraphicsMagick Group
 % Copyright (C) 2002 ImageMagick Studio
 %
 % This program is covered by multiple licenses, which are described in
@@ -46,6 +46,12 @@
 #include "magick/transform.h"
 #include "magick/utility.h"
 
+/*
+  Macro definitions (from Windows wingdi.h).
+*/
+#undef BI_RLE8
+#define BI_RLE8  1
+
 /*
   Typedef declarations.
 */
@@ -76,7 +82,7 @@ typedef struct _DIBInfo
     blue_mask,
     alpha_mask;
 
-  long
+  magick_int32_t
     colorspace;
 
   PointInfo
@@ -107,7 +113,7 @@ static void LogDIBInfo(const DIBInfo *dib_info)
                         "    Bits Per Pixel:       %u\n"
                         "    Compression:          %u\n"
                         "    Size Of Bitmap:       %u\n"
-                        "    Horzontal Resolution: %u\n"
+                        "    Horizontal Resolution:%u\n"
                         "    Vertical Resolution:  %u\n"
                         "    Colors Used:          %u\n"
                         "    Colors Important:     %u",
@@ -156,19 +162,23 @@ static void LogDIBInfo(const DIBInfo *dib_info)
 %    o pixels:  The address of a byte (8 bits) array of pixel data created by
 %      the decoding process.
 %
+%    o pixels_size: The size of the allocated buffer array.
+%
 %
 */
-static unsigned int DecodeImage(Image *image,const unsigned long compression,
-  unsigned char *pixels)
+static MagickPassFail DecodeImage(Image *image,const unsigned long compression,
+                                  unsigned char *pixels, const size_t pixels_size)
 {
-  long
-    byte,
-    count,
+  unsigned long
+    x,
     y;
 
-  register long
-    i,
-    x;
+  unsigned int
+    i;
+
+  int
+    byte,
+    count;
 
   register unsigned char
     *q;
@@ -178,108 +188,175 @@ static unsigned int DecodeImage(Image *image,const unsigned long compression,
 
   assert(image != (Image *) NULL);
   assert(pixels != (unsigned char *) NULL);
-  (void) memset(pixels,0,image->columns*image->rows);
+  if (image->logging)
+    (void) LogMagickEvent(CoderEvent,GetMagickModule(),
+                          "  Decoding RLE compressed pixels to"
+                          " %" MAGICK_SIZE_T_F "u bytes",
+                          image->rows*image->columns);
+
   byte=0;
   x=0;
   q=pixels;
-  end=pixels + (size_t) image->columns*image->rows;
-  for (y=0; y < (long) image->rows; )
-  {
-    if (q < pixels || q  >= end)
-      break;
-    count=ReadBlobByte(image);
-    if (count == EOF)
-      break;
-    if (count != 0)
-      {
-        count=Min(count, end - q);
-        /*
-          Encoded mode.
-        */
-        byte=ReadBlobByte(image);
-        for (i=0; i < count; i++)
+  end=pixels + pixels_size;
+  /*
+    Decompress sufficient data to support the number of pixels (or
+    rows) in the image and then return.
+
+    Do not wait to read the final EOL and EOI markers (if not yet
+    encountered) since we always read this marker just before we
+    return.
+  */
+  for (y=0; y < image->rows; )
+    {
+      if (q < pixels || q >= end)
         {
-          if (compression == 1)
-            *q++=(unsigned char) byte;
-          else
-            *q++=(unsigned char)
-              ((i & 0x01) ? (byte & 0x0f) : ((byte >> 4) & 0x0f));
-          x++;
+          if (image->logging)
+            (void) LogMagickEvent(CoderEvent,GetMagickModule(),
+                                  "  Decode buffer full (y=%lu, "
+                                  "pixels_size=%" MAGICK_SIZE_T_F "u, "
+                                  "pixels=%p, q=%p, end=%p)",
+                                  y, (MAGICK_SIZE_T) pixels_size, pixels, q, end);
+          break;
         }
-      }
-    else
-      {
-        /*
-          Escape mode.
-        */
-        count=ReadBlobByte(image);
-        if (count == 0x01)
-          return(True);
-        switch ((int) count)
+      count=ReadBlobByte(image);
+      if (count == EOF)
+        return MagickFail;
+      if (count > 0)
         {
-          case 0x00:
-          {
-            /*
-              End of line.
-            */
-            x=0;
-            y++;
-            q=pixels+y*image->columns;
-            break;
-          }
-          case 0x02:
-          {
-            /*
-              Delta mode.
-            */
-            x+=ReadBlobByte(image);
-            y+=ReadBlobByte(image);
-            q=pixels+y*image->columns+x;
-            break;
-          }
-          default:
-          {
-            /*
-              Absolute mode.
-            */
-      	    count=Min(count, end - q);
-            for (i=0; i < count; i++)
+          count=Min(count, end - q);
+          /*
+            Encoded mode.
+          */
+          byte=ReadBlobByte(image);
+          if (byte == EOF)
+            return MagickFail;
+          if (compression == BI_RLE8)
             {
-              if (compression == 1)
-                *q++=ReadBlobByte(image);
-              else
+              for ( i=count; i != 0; --i )
+                {
+                  *q++=(unsigned char) byte;
+                }
+            }
+          else
+            {
+              for ( i=0; i < (unsigned int) count; i++ )
                 {
-                  if ((i & 0x01) == 0)
-                    byte=ReadBlobByte(image);
                   *q++=(unsigned char)
                     ((i & 0x01) ? (byte & 0x0f) : ((byte >> 4) & 0x0f));
                 }
-              x++;
             }
-            /*
-              Read pad byte.
-            */
-            if (compression == 1)
-              {
-                if (count & 0x01)
-                  (void) ReadBlobByte(image);
-              }
-            else
-              if (((count & 0x03) == 1) || ((count & 0x03) == 2))
-                (void) ReadBlobByte(image);
-            break;
-          }
+          x+=count;
         }
-      }
-    if (QuantumTick(y,image->rows))
-      if (!MagickMonitorFormatted(y,image->rows,&image->exception,
-                                  LoadImageText,image->filename,
-				  image->columns,image->rows))
-        break;
-  }
+      else
+        {
+          /*
+            Escape mode.
+          */
+          count=ReadBlobByte(image);
+          if (count == EOF)
+            return MagickFail;
+          if (count == 0x01)
+            {
+              if (image->logging)
+                (void) LogMagickEvent(CoderEvent,GetMagickModule(),
+                                      "  RLE Escape code encountered");
+              goto rle_decode_done;
+            }
+          switch (count)
+            {
+            case 0x00:
+              {
+                /*
+                  End of line.
+                */
+                x=0;
+                y++;
+                q=pixels+y*image->columns;
+                break;
+              }
+            case 0x02:
+              {
+                /*
+                  Delta mode.
+                */
+                byte=ReadBlobByte(image);
+                if (byte == EOF)
+                  return MagickFail;
+                x+=byte;
+                byte=ReadBlobByte(image);
+                if (byte == EOF)
+                  return MagickFail;
+                y+=byte;
+                q=pixels+y*image->columns+x;
+                break;
+              }
+            default:
+              {
+                /*
+                  Absolute mode.
+                */
+                count=Min(count, end - q);
+                if (count < 0)
+                  return MagickFail;
+                if (compression == BI_RLE8)
+                  for (i=count; i != 0; --i)
+                    {
+                      byte=ReadBlobByte(image);
+                      if (byte == EOF)
+                        return MagickFail;
+                      *q++=byte;
+                    }
+                else
+                  for (i=0; i < (unsigned int) count; i++)
+                    {
+                      if ((i & 0x01) == 0)
+                        {
+                          byte=ReadBlobByte(image);
+                          if (byte == EOF)
+                            return MagickFail;
+                        }
+                      *q++=(unsigned char)
+                        ((i & 0x01) ? (byte & 0x0f) : ((byte >> 4) & 0x0f));
+                    }
+                x+=count;
+                /*
+                  Read pad byte.
+                */
+                if (compression == BI_RLE8)
+                  {
+                    if (count & 0x01)
+                      if (ReadBlobByte(image) == EOF)
+                        return MagickFail;
+                  }
+                else
+                  if (((count & 0x03) == 1) || ((count & 0x03) == 2))
+                    if (ReadBlobByte(image) == EOF)
+                      return MagickFail;
+                break;
+              }
+            }
+        }
+      if (QuantumTick(y,image->rows))
+        if (!MagickMonitorFormatted(y,image->rows,&image->exception,
+                                    LoadImageText,image->filename,
+                                    image->columns,image->rows))
+          break;
+    }
   (void) ReadBlobByte(image);  /* end of line */
   (void) ReadBlobByte(image);
-  return(True);
+ rle_decode_done:
+  if (image->logging)
+    (void) LogMagickEvent(CoderEvent,GetMagickModule(),
+                          "  Decoded %" MAGICK_SIZE_T_F "u bytes",
+                          (MAGICK_SIZE_T) (q-pixels));
+  if ((MAGICK_SIZE_T) (q-pixels) < pixels_size)
+    {
+      if (image->logging)
+        (void) LogMagickEvent(CoderEvent,GetMagickModule(),
+                              "  RLE decoded output is truncated");
+      return MagickFail;
+    }
+  return(MagickPass);
 }
 
 /*
@@ -365,7 +442,7 @@ static size_t EncodeImage(Image *image,const unsigned long bytes_per_line,
     if (QuantumTick(y,image->rows))
       if (!MagickMonitorFormatted(y,image->rows,&image->exception,
                                   SaveImageText,image->filename,
-				  image->columns,image->rows))
+                                  image->columns,image->rows))
         break;
   }
   /*
@@ -486,7 +563,9 @@ static Image *ReadDIBImage(const ImageInfo *image_info,ExceptionInfo *exception)
     status;
 
   size_t
-    bytes_per_line;
+    bytes_per_line,
+    packet_size,
+    pixels_size;
 
   magick_off_t
     file_size;
@@ -557,6 +636,8 @@ static Image *ReadDIBImage(const ImageInfo *image_info,ExceptionInfo *exception)
       ThrowReaderException(CorruptImageError,NegativeOrZeroImageSize,image);
   if (dib_info.height == 0)
       ThrowReaderException(CorruptImageError,NegativeOrZeroImageSize,image);
+  if (dib_info.height < -2147483647)
+    ThrowReaderException(CorruptImageError,ImproperImageHeader,image);
   image->matte=dib_info.bits_per_pixel == 32;
   image->columns=AbsoluteValue(dib_info.width);
   image->rows=AbsoluteValue(dib_info.height);
@@ -565,9 +646,11 @@ static Image *ReadDIBImage(const ImageInfo *image_info,ExceptionInfo *exception)
     ThrowReaderException(CorruptImageError,ImproperImageHeader,image);
   if (dib_info.colors_important > 256)
     ThrowReaderException(CorruptImageError,ImproperImageHeader,image);
+  if ((dib_info.number_colors != 0) && (dib_info.bits_per_pixel > 8))
+    ThrowReaderException(CorruptImageError,ImproperImageHeader,image);
   if ((dib_info.image_size != 0U) && (dib_info.image_size > file_size))
     ThrowReaderException(CorruptImageError,UnexpectedEndOfFile,image);
-  if ((dib_info.number_colors != 0) || (dib_info.bits_per_pixel < 16))
+  if ((dib_info.number_colors != 0) || (dib_info.bits_per_pixel <= 8))
     {
       image->storage_class=PseudoClass;
       image->colors=dib_info.number_colors;
@@ -638,19 +721,19 @@ static Image *ReadDIBImage(const ImageInfo *image_info,ExceptionInfo *exception)
   /*
     Read image data.
   */
+  packet_size=dib_info.bits_per_pixel;
   if (dib_info.compression == 2)
-    dib_info.bits_per_pixel<<=1;
+    packet_size<<=1;
 
   /*
      Below emulates:
-     bytes_per_line=4*((image->columns*dib_info.bits_per_pixel+31)/32);
+     bytes_per_line=4*((image->columns*dib_info.packet_size+31)/32);
   */
-  bytes_per_line=MagickArraySize(image->columns,dib_info.bits_per_pixel);
-  if (bytes_per_line)
-    bytes_per_line += 31;
-  bytes_per_line /= 32;
-  bytes_per_line=MagickArraySize(4,bytes_per_line);
-
+  bytes_per_line=MagickArraySize(image->columns,packet_size);
+  if ((bytes_per_line > 0) && (~((size_t) 0) - bytes_per_line) > 31)
+    bytes_per_line = MagickArraySize(4,(bytes_per_line+31)/32);
+  if (bytes_per_line == 0)
+    ThrowReaderException(CoderError,ArithmeticOverflow,image);
   (void) LogMagickEvent(CoderEvent,GetMagickModule(),
                         "%" MAGICK_SIZE_T_F "u bytes per line",
                         (MAGICK_SIZE_T) bytes_per_line);
@@ -676,10 +759,13 @@ static Image *ReadDIBImage(const ImageInfo *image_info,ExceptionInfo *exception)
   */
   length=MagickArraySize(bytes_per_line,image->rows);
   if (length == 0)
-    ThrowReaderException(ResourceLimitError,MemoryAllocationFailed,image);
-  pixels=MagickAllocateArray(unsigned char *,
-                             image->rows,
-                             Max(bytes_per_line,image->columns+1));
+    ThrowReaderException(CoderError,ArithmeticOverflow,image);
+  if (~((size_t) 0) - image->columns < 1)
+    ThrowReaderException(CoderError,ArithmeticOverflow,image);
+  pixels_size=MagickArraySize(image->rows,Max(bytes_per_line,(size_t) image->columns+1));
+  if (pixels_size == 0)
+    ThrowReaderException(CoderError,ArithmeticOverflow,image);
+  pixels=MagickAllocateMemory(unsigned char *,pixels_size);
   if (pixels == (unsigned char *) NULL)
     ThrowReaderException(ResourceLimitError,MemoryAllocationFailed,image);
   if ((dib_info.compression == 0) || (dib_info.compression == 3))
@@ -700,9 +786,12 @@ static Image *ReadDIBImage(const ImageInfo *image_info,ExceptionInfo *exception)
     {
       /*
         Convert run-length encoded raster pixels.
-        DecodeImage expects that pixels array is rows*columns bytes.
+
+        DecodeImage() normally decompresses to rows*columns bytes of data.
       */
-      status=DecodeImage(image,dib_info.compression,pixels);
+      (void) memset(pixels,0,pixels_size);
+      status=DecodeImage(image,dib_info.compression,pixels,
+                         image->rows*image->columns);
       if (status == False)
         {
           MagickFreeMemory(pixels);
@@ -763,7 +852,7 @@ static Image *ReadDIBImage(const ImageInfo *image_info,ExceptionInfo *exception)
               status=MagickMonitorFormatted(image->rows-y-1,image->rows,
                                             exception,LoadImageText,
                                             image->filename,
-					    image->columns,image->rows);
+                                            image->columns,image->rows);
               if (status == False)
                 break;
             }
@@ -810,7 +899,7 @@ static Image *ReadDIBImage(const ImageInfo *image_info,ExceptionInfo *exception)
               status=MagickMonitorFormatted(image->rows-y-1,image->rows,
                                             exception,LoadImageText,
                                             image->filename,
-					    image->columns,image->rows);
+                                            image->columns,image->rows);
               if (status == False)
                 break;
             }
@@ -848,7 +937,7 @@ static Image *ReadDIBImage(const ImageInfo *image_info,ExceptionInfo *exception)
               status=MagickMonitorFormatted(image->rows-y-1,image->rows,
                                             exception,LoadImageText,
                                             image->filename,
-					    image->columns,image->rows);
+                                            image->columns,image->rows);
               if (status == False)
                 break;
             }
@@ -861,7 +950,7 @@ static Image *ReadDIBImage(const ImageInfo *image_info,ExceptionInfo *exception)
         word;
 
       /*
-        Convert PseudoColor scanline.
+        Convert DirectColor (555 or 565) scanline.
       */
       image->storage_class=DirectClass;
       if (dib_info.compression == 1)
@@ -898,7 +987,7 @@ static Image *ReadDIBImage(const ImageInfo *image_info,ExceptionInfo *exception)
               status=MagickMonitorFormatted(image->rows-y-1,image->rows,
                                             exception,LoadImageText,
                                             image->filename,
-					    image->columns,image->rows);
+                                            image->columns,image->rows);
               if (status == False)
                 break;
             }
@@ -934,7 +1023,7 @@ static Image *ReadDIBImage(const ImageInfo *image_info,ExceptionInfo *exception)
               status=MagickMonitorFormatted(image->rows-y-1,image->rows,
                                             exception,LoadImageText,
                                             image->filename,
-					    image->columns,image->rows);
+                                            image->columns,image->rows);
               if (status == False)
                 break;
             }
@@ -959,34 +1048,50 @@ static Image *ReadDIBImage(const ImageInfo *image_info,ExceptionInfo *exception)
       char
         byte;
 
-      image->storage_class=DirectClass;
-      image->matte=True;
+      image->matte=MagickFalse;
       for (y=(long) image->rows-1; y >= 0; y--)
         {
+          if (image->logging)
+            (void) LogMagickEvent(CoderEvent,GetMagickModule(),
+                                  "y=%ld", y);
           q=GetImagePixels(image,0,y,image->columns,1);
           if (q == (PixelPacket *) NULL)
             break;
           for (x=0; x < ((long) image->columns-7); x+=8)
             {
               byte=0;
-              (void) ReadBlob(image,sizeof(byte),&byte);
+              if (ReadBlob(image,sizeof(byte),&byte) != sizeof(byte))
+                break;
               for (bit=0; bit < 8; bit++)
-                q[x+bit].opacity=(Quantum)
-                  (byte & (0x80 >> bit) ? TransparentOpacity : OpaqueOpacity);
+                {
+                  q[x+bit].opacity=(Quantum)
+                    (byte & (0x80 >> bit) ? TransparentOpacity : OpaqueOpacity);
+                  if (q[x+bit].opacity != OpaqueOpacity)
+                    image->matte=MagickTrue;
+                }
             }
+          /* Detect early loop termination above due to EOF */
+          if (x < ((long) image->columns-7))
+            break;
           if ((image->columns % 8) != 0)
             {
               byte=0;
-              (void) ReadBlob(image,sizeof(byte),&byte);
+              if (ReadBlob(image,sizeof(byte),&byte) != sizeof(byte))
+                break;
               for (bit=0; bit < (long) (image->columns % 8); bit++)
-                q[x+bit].opacity=(Quantum)
-                  (byte & (0x80 >> bit) ? TransparentOpacity : OpaqueOpacity);
+                {
+                  q[x+bit].opacity=(Quantum)
+                    (byte & (0x80 >> bit) ? TransparentOpacity : OpaqueOpacity);
+                  if (q[x+bit].opacity != OpaqueOpacity)
+                    image->matte=MagickTrue;
+                }
             }
           if (image->columns % 32)
             for (x=0; x < (long) ((32-(image->columns % 32))/8); x++)
               {
                 byte=0;
-                (void) ReadBlob(image,sizeof(byte),&byte);
+                if (ReadBlob(image,sizeof(byte),&byte) != sizeof(byte))
+                  break;
               }
           if (!SyncImagePixels(image))
             break;
@@ -997,9 +1102,24 @@ static Image *ReadDIBImage(const ImageInfo *image_info,ExceptionInfo *exception)
                                           image->columns,image->rows))
                 break;
         }
+      /*
+        If a PseudoClass image has a non-opaque opacity channel, then
+        we must mark it as DirectClass since there is no standard way
+        to store PseudoClass with an opacity channel.
+      */
+      if ((image->storage_class == PseudoClass) && (image->matte == MagickTrue))
+        image->storage_class=DirectClass;
+#if 0
+      /*
+        FIXME: SourceForge bug 557 provides an icon for which magick
+        is set to "ICODIB" by the 'icon' coder but there is no data
+        for the ICO mask.  Intentionally ignore EOF at this point
+        until this issue gets figured out.
+       */
       if (EOFBlob(image))
         ThrowException(exception,CorruptImageError,UnexpectedEndOfFile,
                        image->filename);
+#endif
     }
   if (dib_info.height < 0)
     {
@@ -1209,7 +1329,15 @@ static unsigned int WriteDIBImage(const ImageInfo *image_info,Image *image)
         dib_info.bits_per_pixel=1;
       dib_info.number_colors=1 << dib_info.bits_per_pixel;
     }
-  bytes_per_line=4*((image->columns*dib_info.bits_per_pixel+31)/32);
+  /*
+     Below emulates:
+     bytes_per_line=4*((image->columns*dib_info.bits_per_pixel+31)/32);
+  */
+  bytes_per_line=MagickArraySize(image->columns,dib_info.bits_per_pixel);
+  if ((bytes_per_line > 0) && (~((size_t) 0) - bytes_per_line) > 31)
+    bytes_per_line = MagickArraySize(4,(bytes_per_line+31)/32);
+  if (bytes_per_line == 0)
+    ThrowWriterException(CoderError,ArithmeticOverflow,image);
   dib_info.header_size=40;
   dib_info.width=(long) image->columns;
   dib_info.height=(long) image->rows;
@@ -1277,7 +1405,7 @@ static unsigned int WriteDIBImage(const ImageInfo *image_info,Image *image)
          if (QuantumTick(y,image->rows))
            if (!MagickMonitorFormatted(y,image->rows,&image->exception,
                                        SaveImageText,image->filename,
-				       image->columns,image->rows))
+                                       image->columns,image->rows))
              break;
       }
       break;
@@ -1306,7 +1434,7 @@ static unsigned int WriteDIBImage(const ImageInfo *image_info,Image *image)
           if (QuantumTick(y,image->rows))
             if (!MagickMonitorFormatted(y,image->rows,&image->exception,
                                         SaveImageText,image->filename,
-					image->columns,image->rows))
+                                        image->columns,image->rows))
               break;
       }
       break;
@@ -1343,7 +1471,7 @@ static unsigned int WriteDIBImage(const ImageInfo *image_info,Image *image)
           if (QuantumTick(y,image->rows))
             if (!MagickMonitorFormatted(y,image->rows,&image->exception,
                                         SaveImageText,image->filename,
-					image->columns,image->rows))
+                                        image->columns,image->rows))
                break;
       }
       break;
