@@ -1,5 +1,5 @@
 /*
-% Copyright (C) 2003 - 2012 GraphicsMagick Group
+% Copyright (C) 2003-2018 GraphicsMagick Group
 % Copyright (C) 2002 ImageMagick Studio
 % Copyright 1991-1999 E. I. du Pont de Nemours and Company
 %
@@ -39,12 +39,27 @@
 #include "magick/studio.h"
 #include "magick/utility.h"
 
+#if defined(MAGICK_MEMORY_HARD_LIMIT)
+#define MEMORY_LIMIT_CHECK(func,size)                                   \
+  do                                                                    \
+    {                                                                   \
+      if (0) fprintf(stderr,"%s: %zu\n", func, size);                   \
+      if (size > (size_t)MAGICK_MEMORY_HARD_LIMIT)                      \
+        {                                                               \
+          fprintf(stderr,"%s: Excessive allocation request %zu\n", func, size); \
+          abort();                                                      \
+        }                                                               \
+  } while(0)
+#else
+#define MEMORY_LIMIT_CHECK(func,size)
+#endif /* MAGICK_MEMORY_HARD_LIMIT */
+
 /*
   Static variables.
 */
-MagickFreeFunc    FreeFunc    = free;
-MagickMallocFunc  MallocFunc  = malloc;
-MagickReallocFunc ReallocFunc = realloc;
+static MagickFreeFunc    FreeFunc    = free;
+static MagickMallocFunc  MallocFunc  = malloc;
+static MagickReallocFunc ReallocFunc = realloc;
 
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
@@ -59,7 +74,8 @@ MagickReallocFunc ReallocFunc = realloc;
 %
 %  MagickAllocFunctions() provides a way for the user to supply a preferred
 %  free(), malloc(), and realloc() functions.  Otherwise the default system
-%  versions are used.
+%  versions are used.  If an alternative allocator is to be used, this
+%  function should be invoked prior to invoking InitializeMagick().
 %
 %  The format of the  MagickAllocFunctions method is:
 %
@@ -152,6 +168,9 @@ MagickExport void * MagickMalloc(const size_t size)
   if (size == 0)
     return ((void *) NULL);
 
+  /* fprintf(stderr,"%s %zu\n",__func__, size); */
+  MEMORY_LIMIT_CHECK(GetCurrentFunction(),size);
+
   return (MallocFunc)(size);
 }
 
@@ -174,20 +193,27 @@ MagickExport void * MagickMalloc(const size_t size)
 %  alignment and ending at at least the requested alignment.
 %
 %  The requested alignment should be a power of 2 at least as large as
-%  sizeof(void *).
-% 
+%  sizeof a void pointer.
+%
 %  NULL is returned if insufficient memory is available, the requested
 %  size is zero, or integer overflow was detected.
 %
 %  This function is intended for allocating special-purpose buffers
 %  which benefit from specific alignment.
 %
+%  The allocated memory should only be freed using MagickFreeAligned()
+%  and may not be reallocated.
+%
 %  The format of the  MagickMallocAligned method is:
 %
 %      void * MagickMallocAligned(size_t alignment, const size_t size)
 %
 %  A description of each parameter follows:
 %
+%
+%    o alignment: The alignment of the base and size of the allocated
+%                 memory.
+%
 %    o size: The size of the memory in bytes to allocate.
 %
 %
@@ -200,6 +226,9 @@ MagickExport void * MagickMallocAligned(const size_t alignment,const size_t size
   void
     *alligned_p = 0;
 
+  /* fprintf(stderr,"%s %zu\n",__func__, size); */
+  MEMORY_LIMIT_CHECK(GetCurrentFunction(),size);
+
   alligned_size=RoundUpToAlignment(size,alignment);
 
   if ((size == 0) || (alignment < sizeof(void *)) || (alligned_size < size))
@@ -221,12 +250,12 @@ MagickExport void * MagickMallocAligned(const size_t alignment,const size_t size
     alloc_size=(size+alignment-1)+sizeof(void *);
     if (alloc_size > size)
       {
-	if ((alloc_p = (MallocFunc)(alloc_size)) != NULL)
-	  {
-	    alligned_p=(void*) RoundUpToAlignment((magick_uintptr_t)alloc_p+
-						  sizeof(void *),alignment);
-	    *((void **)alligned_p-1)=alloc_p;
-	  }
+        if ((alloc_p = (MagickMalloc(alloc_size))) != NULL)
+          {
+            alligned_p=(void*) RoundUpToAlignment((magick_uintptr_t)alloc_p+
+                                                  sizeof(void *),alignment);
+            *((void **)alligned_p-1)=alloc_p;
+          }
       }
   }
 #endif
@@ -239,6 +268,68 @@ MagickExport void * MagickMallocAligned(const size_t alignment,const size_t size
 %                                                                             %
 %                                                                             %
 %                                                                             %
++   M a g i c k M a l l o c A l i g n e d A r r a y                           %
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%
+%  MagickMallocAlignedArray() returns a pointer to a block of memory of
+%  sufficient size to support an array of elements of a specified size.
+%  The allocation's base address is an even multiple of the specified
+%  alignment.  The size of the buffer allocation is rounded up as required
+%  in order to consume a block of memory starting at least at the requested
+%  alignment and ending at at least the requested alignment.
+%
+%  NULL is returned if the required memory exceeds the range of size_t,
+%  the computed size is zero, or there is insufficient memory available.
+%
+%  This function is intended for allocating special-purpose buffers
+%  which benefit from specific alignment.
+%
+%  The allocated memory should only be freed using MagickFreeAligned()
+%  and may not be reallocated.
+%
+%  The format of the MagickMallocArray method is:
+%
+%      void *MagickMallocAlignedArray(const size_t alignment,
+%                                     const size_t count,
+%                                     const size_t size);
+%
+%  A description of each parameter follows:
+%
+%    o alignment: The alignment of the base and size of the allocated
+%                 memory.
+%
+%    o count: The number of elements in the array.
+%
+%    o size: The size of one array element.
+%
+*/
+MagickExport void *MagickMallocAlignedArray(const size_t alignment,
+                                            const size_t count,
+                                            const size_t size)
+{
+  size_t
+    allocation_size;
+
+  void
+    *allocation;
+
+  allocation = (void *) NULL;
+  allocation_size=MagickArraySize(count,size);
+
+  if (allocation_size)
+    allocation = MagickMallocAligned(alignment,allocation_size);
+
+  return allocation;
+}
+
+/*
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%                                                                             %
+%                                                                             %
+%                                                                             %
 +   M a g i c k M a l l o c A r r a y                                         %
 %                                                                             %
 %                                                                             %
@@ -274,7 +365,7 @@ MagickExport void *MagickMallocArray(const size_t count,const size_t size)
   allocation_size=MagickArraySize(count,size);
 
   if (allocation_size)
-    allocation = (MallocFunc)(allocation_size);
+    allocation = MagickMalloc(allocation_size);
   return allocation;
 }
 
@@ -311,10 +402,10 @@ MagickExport void * MagickMallocCleared(const size_t size)
 
   if (size != 0)
     {
-      p=(MallocFunc)(size);
+      p=MagickMalloc(size);
 
       if (p != (void *) NULL)
-	(void) memset(p,0,size);
+        (void) memset(p,0,size);
     }
 
   return p;
@@ -397,12 +488,14 @@ MagickExport void *MagickRealloc(void *memory,const size_t size)
   void
     *new_memory = (void *) NULL;
 
+  MEMORY_LIMIT_CHECK(GetCurrentFunction(),size);
+
   if ((void *) NULL == memory)
     new_memory = (MallocFunc)(size);
   else
     new_memory = (ReallocFunc)(memory,size);
   if ((new_memory == 0) && (memory != 0) && (size != 0))
-    (FreeFunc)(memory);
+    MagickFree(memory);
 
   return new_memory;
 }
@@ -472,12 +565,12 @@ MagickExport void MagickFreeAligned(void *memory)
   if (memory != (void *) NULL)
     {
 #if defined (HAVE_POSIX_MEMALIGN)
-      (FreeFunc)(memory);
+      MagickFree(memory);
 #elif defined(HAVE__ALIGNED_MALLOC)
       /* Windows Studio .NET 2003 or later */
       _aligned_free(memory);
 #else
-      (FreeFunc)(*((void **)memory-1));
+      MagickFree(*((void **)memory-1));
 #endif
     }
 }
