@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2003 - 2013 GraphicsMagick Group
+ * Copyright (C) 2003 - 2018 GraphicsMagick Group
  * Copyright (C) 2003 ImageMagick Studio
  * Copyright 1991-1999 E. I. du Pont de Nemours and Company
  *
@@ -19,6 +19,7 @@
  */
 
 #include <magick/api.h>
+#include <magick/enum_strings.h>
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -74,32 +75,14 @@ int main ( int argc, char **argv )
     {
       char
         *option = argv[arg];
-    
+
       if (*option == '-')
         {
           if (LocaleCompare("compress",option+1) == 0)
             {
               arg++;
               option=argv[arg];
-              imageInfo->compression=UndefinedCompression;
-              if (LocaleCompare("None",option) == 0)
-                imageInfo->compression=NoCompression;
-              if (LocaleCompare("BZip",option) == 0)
-                imageInfo->compression=BZipCompression;
-              if (LocaleCompare("Fax",option) == 0)
-                imageInfo->compression=FaxCompression;
-              if (LocaleCompare("Group4",option) == 0)
-                imageInfo->compression=Group4Compression;
-              if (LocaleCompare("JPEG",option) == 0)
-                imageInfo->compression=JPEGCompression;
-              if (LocaleCompare("Lossless",option) == 0)
-                imageInfo->compression=LosslessJPEGCompression;
-              if (LocaleCompare("LZW",option) == 0)
-                imageInfo->compression=LZWCompression;
-              if (LocaleCompare("RLE",option) == 0)
-                imageInfo->compression=RLECompression;
-              if (LocaleCompare("Zip",option) == 0)
-                imageInfo->compression=ZipCompression;
+              imageInfo->compression=StringToCompressionType(option);
             }
           else if (LocaleCompare("debug",option+1) == 0)
             {
@@ -117,10 +100,10 @@ int main ( int argc, char **argv )
                   goto program_exit;
                 }
               if(imageInfo->depth != 8 && imageInfo->depth != 16 &&
-		 imageInfo->depth != 32)
+                 imageInfo->depth != 32)
                 {
                   (void) printf("-depth (%ld) not 8, 16, or 32\n",
-				imageInfo->depth);
+                                imageInfo->depth);
                   (void) fflush(stdout);
                   exit_status = 1;
                   goto program_exit;
@@ -184,9 +167,9 @@ int main ( int argc, char **argv )
     {
       (void) printf("arg=%d, argc=%d\n", arg, argc);
       (void) printf ( "Usage: %s [-compress algorithm -debug events -depth "
-		      "integer -define value -log format -nocheck -quality quality "
+                      "integer -define value -log format -nocheck -quality quality "
                       "-size geometry -verbose] "
-		      "infile format\n", argv[0] );
+                      "infile format\n", argv[0] );
       (void) fflush(stdout);
       exit_status = 1;
       goto program_exit;
@@ -231,6 +214,8 @@ int main ( int argc, char **argv )
   original = ReadImage ( imageInfo, &exception );
   if (exception.severity != UndefinedException)
     {
+      if (exception.reason)
+        (void) printf("    reason:%s\n",exception.reason);
       CatchException(&exception);
       exit_status = 1;
       goto program_exit;
@@ -238,7 +223,7 @@ int main ( int argc, char **argv )
   if ( original == (Image *)NULL )
     {
       (void) printf ( "Failed to read original image %s\n",
-		      imageInfo->filename );
+                      imageInfo->filename );
       (void) fflush(stdout);
       exit_status = 1;
       goto program_exit;
@@ -270,14 +255,24 @@ int main ( int argc, char **argv )
   blob =(char *) ImageToBlob ( imageInfo, original, &blob_length, &exception );
   if (exception.severity != UndefinedException)
     {
+      if (exception.reason)
+        (void) printf("    reason:%s\n",exception.reason);
       CatchException(&exception);
       exit_status = 1;
       goto program_exit;
     }
   if ( blob == NULL )
     {
-      (void) printf ( "Failed to write BLOB in format %s\n",
-		      imageInfo->magick );
+      (void) printf ( "Failed to write BLOB in format %s (blob is NULL!)\n",
+                      imageInfo->magick );
+      (void) fflush(stdout);
+      exit_status = 1;
+      goto program_exit;
+    }
+  if ( blob_length == 0)
+    {
+      (void) printf ( "Failed to write BLOB in format %s (blob length is 0!)\n",
+                      imageInfo->magick );
       (void) fflush(stdout);
       exit_status = 1;
       goto program_exit;
@@ -303,28 +298,30 @@ int main ( int argc, char **argv )
     ping_image = PingBlob(imageInfo, blob, blob_length, &exception );
     if (exception.severity != UndefinedException)
       {
-	CatchException(&exception);
-	(void) fflush(stderr);
-	ping_error = MagickTrue;
+        if (exception.reason)
+          (void) printf("    reason:%s\n",exception.reason);
+        CatchException(&exception);
+        (void) fflush(stderr);
+        ping_error = MagickTrue;
       }
     if ( ping_image == (Image *)NULL )
       {
-	(void) printf ( "Failed to ping image from BLOB in format %s\n",
-			imageInfo->magick );
-	(void) fflush(stdout);
-	ping_error = MagickTrue;
+        (void) printf ( "Failed to ping image from BLOB in format %s\n",
+                        imageInfo->magick );
+        (void) fflush(stdout);
+        ping_error = MagickTrue;
       }
     else
       {
-	/* Print a short description of the image to stdout */
-	DescribeImage( ping_image, stdout, MagickFalse );
-	(void) fflush(stdout);
-	DestroyImageList( ping_image );
+        /* Print a short description of the image to stdout */
+        DescribeImage( ping_image, stdout, MagickFalse );
+        (void) fflush(stdout);
+        DestroyImageList( ping_image );
       }
     if (ping_error)
       {
-	exit_status = 1;
-	goto program_exit;
+        exit_status = 1;
+        goto program_exit;
       }
   }
 
@@ -338,6 +335,8 @@ int main ( int argc, char **argv )
   original = BlobToImage( imageInfo, blob, blob_length, &exception );
   if (exception.severity != UndefinedException)
     {
+      if (exception.reason)
+        (void) printf("    reason:%s\n",exception.reason);
       CatchException(&exception);
       exit_status = 1;
       goto program_exit;
@@ -345,7 +344,7 @@ int main ( int argc, char **argv )
   if ( original == (Image *)NULL )
     {
       (void) printf ( "Failed to read image from BLOB in format %s\n",
-		      imageInfo->magick );
+                      imageInfo->magick );
       (void) fflush(stdout);
       exit_status = 1;
       goto program_exit;
@@ -361,9 +360,11 @@ int main ( int argc, char **argv )
   (void) strcpy( imageInfo->filename, "" );
   original->delay = 10;
   blob = (char *) ImageToBlob ( imageInfo, original, &blob_length,
-				&exception );
+                                &exception );
   if (exception.severity != UndefinedException)
     {
+      if (exception.reason)
+        (void) printf("    reason:%s\n",exception.reason);
       CatchException(&exception);
       exit_status = 1;
       goto program_exit;
@@ -372,7 +373,7 @@ int main ( int argc, char **argv )
   if ( blob == NULL )
     {
       (void) printf ( "Failed to write BLOB in format %s\n",
-		      imageInfo->magick );
+                      imageInfo->magick );
       (void) fflush(stdout);
       exit_status = 1;
       goto program_exit;
@@ -390,6 +391,8 @@ int main ( int argc, char **argv )
   final = BlobToImage( imageInfo, blob, blob_length, &exception );
   if (exception.severity != UndefinedException)
     {
+      if (exception.reason)
+        (void) printf("    reason:%s\n",exception.reason);
       CatchException(&exception);
       exit_status = 1;
       goto program_exit;
@@ -397,7 +400,7 @@ int main ( int argc, char **argv )
   if ( final == (Image *)NULL )
     {
       (void) printf ( "Failed to read image from BLOB in format %s\n",
-		      imageInfo->magick );
+                      imageInfo->magick );
       (void) fflush(stdout);
       exit_status = 1;
       goto program_exit;
@@ -411,10 +414,10 @@ int main ( int argc, char **argv )
        * Check final output
        */
       double
-	fuzz_factor = 0;
+        fuzz_factor = 0;
 
       /*
-	Some formats are lossy.
+        Some formats are lossy.
       */
       if ((!strcmp( "CIN", format ) && (QuantumDepth == 8)) ||
           (!strcmp( "CMYK", format )) ||
@@ -433,76 +436,90 @@ int main ( int argc, char **argv )
           (final->compression == JPEGCompression))
         fuzz_factor = 0.06;
 
+      switch(imageInfo->compression)
+        {
+        case JPEGCompression:
+        case JPEG2000Compression:
+        case WebPCompression:
+          {
+            fuzz_factor = 0.06;
+            break;
+          }
+        default:
+          {
+          }
+        }
+
       {
-	Image
-	  *o,
-	  *f;
-	
-	unsigned long
-	  frame=0;
+        Image
+          *o,
+          *f;
 
-	/*
-	  Verify that frame pixels are identical (or close enough).
-	*/
-	for (o=original, f=final, frame=0;
-	     ((o != (Image *) NULL) && (f != (Image *) NULL)) ;
-	     o = o->next, f = f->next, frame++)
-	  {
-	    printf("Checking frame %ld...\n",frame);
-	    if ( !IsImagesEqual(o, f ) &&
-		 (original->error.normalized_mean_error > fuzz_factor) )
-	      {
-		(void) printf( "R/W file check for format \"%s\" failed "
-			       "(frame = %ld): %.6f/%.6f/%.6fe\n",
-			       format,frame,
-			       original->error.mean_error_per_pixel,
-			       original->error.normalized_mean_error,
-			       original->error.normalized_maximum_error);
-		(void) fflush(stdout);
-		exit_status = 1;
-		goto program_exit;
-	      }
-	  }
+        unsigned long
+          frame=0;
 
-	if (check_for_added_frames)
-	  {
-	    /*
-	      Verify that reads from BLOB R/W #1 and BLOB R/W #2 did
-	      return the same number of frames.
-	    */
-	    if ((o != (Image *) NULL) && (f != (Image *) NULL))
-	      {
-		(void) printf("R/W file check for format \"%s\" failed due to "
-			      "differing number of returned frames (%ld vs %ld)\n",
-			      format,
-			      GetImageListLength(original),
-			      GetImageListLength(final));
-		exit_status = 1;
-		goto program_exit;
-	      }
+        /*
+          Verify that frame pixels are identical (or close enough).
+        */
+        for (o=original, f=final, frame=0;
+             ((o != (Image *) NULL) && (f != (Image *) NULL)) ;
+             o = o->next, f = f->next, frame++)
+          {
+            printf("Checking frame %ld...\n",frame);
+            if ( !IsImagesEqual(o, f ) &&
+                 (original->error.normalized_mean_error > fuzz_factor) )
+              {
+                (void) printf( "R/W file check for format \"%s\" failed "
+                               "(frame = %ld): %.6f/%.6f/%.6fe\n",
+                               format,frame,
+                               original->error.mean_error_per_pixel,
+                               original->error.normalized_mean_error,
+                               original->error.normalized_maximum_error);
+                (void) fflush(stdout);
+                exit_status = 1;
+                goto program_exit;
+              }
+          }
 
-	    /*
-	      If format supports multiple frames, then we should expect
-	      that frames are not lost (or spuriously added) due to
-	      read/write of format.
-	    */
-	    if (magick_info->adjoin)
-	      {
-		unsigned long
-		  final_frames;
+        if (check_for_added_frames)
+          {
+            /*
+              Verify that reads from BLOB R/W #1 and BLOB R/W #2 did
+              return the same number of frames.
+            */
+            if ((o != (Image *) NULL) && (f != (Image *) NULL))
+              {
+                (void) printf("R/W file check for format \"%s\" failed due to "
+                              "differing number of returned frames (%ld vs %ld)\n",
+                              format,
+                              GetImageListLength(original),
+                              GetImageListLength(final));
+                exit_status = 1;
+                goto program_exit;
+              }
 
-		final_frames=GetImageListLength(final);
-		if (original_frames != final_frames)
-		  {
-		    (void) printf("R/W file check for format \"%s\" failed due "
-				  "to differing number of returned frames (%ld "
-				  "vs %ld) from original file\n",
-				  format,original_frames,final_frames);
-		    exit_status = 1;
-		    goto program_exit;
-		  }
-	      }
-	  }
+            /*
+              If format supports multiple frames, then we should expect
+              that frames are not lost (or spuriously added) due to
+              read/write of format.
+            */
+            if (magick_info->adjoin)
+              {
+                unsigned long
+                  final_frames;
+
+                final_frames=GetImageListLength(final);
+                if (original_frames != final_frames)
+                  {
+                    (void) printf("R/W file check for format \"%s\" failed due "
+                                  "to differing number of returned frames (%ld "
+                                  "vs %ld) from original file\n",
+                                  format,original_frames,final_frames);
+                    exit_status = 1;
+                    goto program_exit;
+                  }
+              }
+          }
       }
     }
 
